-------------[[[[[Mario Engine]]]]]-------------
-------------[[[[[Written by dan369]]]]]-------------
-------------[[[[[For PSP]]]]]-------------

--[[
This is a platformer engine that i'm writing to make a platform game, i want it to be as re-usable as possible. I call it the Mario Engine because
Mario is the good of all platforming games, it set the standard. So thats the reason behind the name choice. It will be written for the PGE API.
It will use Tilemaps, and tile-based collision for the basic features. Another aspect is to make it as easy as possible to create a level.
]]--

----Load Tilesheet
tilesheet = pge.texture.load("tileset.png", PGE_RAM);
miniBlock = pge.texture.load("mini-block.png", PGE_RAM);

----Intial variables used for Tilemap
scr_left = 0; ---screen left offset
scr_top = 0;  ---screen top offset
currentFrame = 1; ---current frame
frameSkip = 2; ---frameskip
tsize = 16; ---tile size
screeNum = 0;  ---screenshot number
maxTilesPerRow = (tilesheet:width() / tsize) - 1; 

--Enemie constants
enemyTypeGoomba = 4;
enemyTypeKoopa = 5;

--Create table to hold mini-block info.
miniBlocks = {};

--Create color
backCol = pge.gfx.createcolor(92, 148, 252); 

----Animation variables, for our animations
--Constants again, interchange depending on current postition/status.
Right = 1;   
Left = 0;
StandingL = 0; 
StandingR = 1;
JumpingR = 8;
JumpingL = 9;
WalkingR = 2;
WalkingL = 5;
Dead = 10;
direction = Right;  ---holds the current direction
currentSpirte = StandingR;  ---holds the current spirte (where to start clipping from)
currentSpirteEnd = 15;    ---this is the width of our image(to end the clipping) i do it manually because of drifferent sized spirtes.

--Mario Animation tables, needed for my animation mini-libray.
MarioAnimRight = {AnimTime = 0, startFrame = WalkingR, endFrame = 5, oldFrame = WalkingR};   ---right animation table
MarioAnimLeft = {AnimTime = 0, startFrame = WalkingL, endFrame = 8, oldFrame = WalkingL};    ---left animation table
MaxTime = 7;   ---MaxTime between frames.
offset = 0;  ---offset required for certain images.

--Function to create the 'player' table.
function createPlayerObject(X, Y, Width, Height, SpeedX, SpeedY, Jump, Img)
	if playerChose == 0 then
		Img = pge.texture.load("mario.png", PGE_RAM);
	elseif playerChose == 1 then
		Img = pge.texture.load("luigi.png", PGE_RAM);
	end
	
	player = {};
	player = {x =  X, y = Y, img = Img, width = Width, height = Height, speedX = SpeedX, speedY = SpeedY, jump = Jump, dead = false};
end

--Function to load a Map
function loadMap(map)
	dofile(map);
end

--Function to load a Collision File
function loadCollisionMap(map)
	dofile(map);
end

--Function to create the 'enemie' objects array. Note: Must be called AFTER loading the collision map! >.>
function createEnemieObjects(colMap)
colMap = layer2 or colMap;

	--load enemie spritesheets
	goomba = pge.texture.load("goomba.png", PGE_RAM);
	--create table constructor
	enemies = {};
	--loop through file, inserting enemie objects into the array.
	for i, row in ipairs(colMap) do
		for j, tile in ipairs(row) do
			if tile == 4 then
				table.insert(enemies, {x = tsize * (j-1), y = (tsize * (i-1))-2, width = tsize, height = 18, speedX = 1, speedY = 0, direction = 0, jump = false, currentImg = goomba, AnimTime = 0, startFrame = 0, endFrame = 2, oldFrame = 0, Type = enemyTypeGoomba, alive = true, counter =0});
				--perform clean-up
				colMap[i][j] = 0;
			end
		end
	end
end

--Function to detect Movement/Jumping
function detectButtons(object)
	if pge.controls.held(PGE_CTRL_RIGHT) or pge.controls.analogx() > 99 then
		direction = Right;
		movePlayer(object, 1, 0);
	elseif pge.controls.held(PGE_CTRL_LEFT) or pge.controls.analogx() < -99 then
		direction = Left;
		movePlayer(object, -1, 0);
	end
	
	if pge.controls.pressed(PGE_CTRL_CROSS) and not object.jump then
		object.jump = true;
		object.speedY = 8.5;
	end
	
	if object.jump then
		jump(object);
	end
end

--Function to get the object corners used in Collision
function getObjectCorners(x, y, object)
	down = math.floor((y + object.height - 1) / tsize)+1;
	up = math.floor(y / tsize)+1;
	left = math.floor(x / tsize)+1;
	right = math.floor((x + object.width - 1) / tsize)+1;
end

--Function to update The Player's X/Y postition
function movePlayer(object, dirx, diry, collisionmap, Map)
collisionmap = layer2;
Map = map;
local collide = false;
local collide1 = false;

if not object.dead then
	getObjectCorners(object.x+object.speedX*dirx, object.y, object)
	if (dirx == 1) then
		if (collisionmap[down][right] == 0 and collisionmap[up][right] == 0) then
			object.x = object.x + (dirx * object.speedX);
			fall(object);
		else
			object.x = ((right-1) * tsize) - object.width;
		end	
	end
	if (dirx == -1) then
		if (collisionmap[down][left] == 0 and collisionmap[up][left] == 0) then	
			object.x = object.x + (dirx * object.speedX);
			fall(object);
		else
			object.x = (left * tsize);
		end
	end
	
	if (diry == 1) then
		    getObjectCorners(object.x, object.y-object.speedY*diry, object);
		if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then	
			object.y = object.y - object.speedY*diry;
		else
			object.y = ((down-1) * tsize) - object.height;
			object.jump = false;
			object.speedY = 0;
		end
	end
	if (diry == -1) then
		    getObjectCorners(object.x, object.y+object.speedY*diry, object);
		if (collisionmap[up][left] == 0 and collisionmap[up][right] == 0) then		
			object.y = object.y + object.speedY*diry;
		else
			object.y = (up * tsize);
			object.speedY = -1;
		end

			if (collisionmap[up][right] == 3) then	
				Map[up][right] = 0;		
				collisionmap[up][right] = 0;
				collide1 = true;
			elseif (collisionmap[up][left] == 3) then
				Map[up][left] = 0;
				collisionmap[up][left] = 0;
				collide = true;
			else
				collide = false;
				collide1 = false;
			end
		end
			
		if collide then
			getObjectCorners(object.x, object.y, object);
			table.insert(miniBlocks, {x = ((left-1)*tsize), y = (up-2)*tsize, img = miniBlock, speedY = 0});  --top-left
			table.insert(miniBlocks, {x = ((left-1)*tsize+10), y = (up-2)*tsize, img = miniBlock, speedY = 0}); --top-right
			table.insert(miniBlocks, {x = ((left-1)*tsize+10), y = ((up-1)*tsize-6), img = miniBlock, speedY = 0}); --bottom-right
			table.insert(miniBlocks, {x = ((left-1)*tsize), y = ((up-1)*tsize-6), img = miniBlock, speedY = 0}); --bottom-left
		elseif collide1 then
			getObjectCorners(object.x, object.y, object);
			table.insert(miniBlocks, {x = ((right-1)*tsize)-4, y = (up-2)*tsize, img = miniBlock, speedY = 0}); --top-left
			table.insert(miniBlocks, {x = ((right-1)*tsize+10)-4, y = (up-2)*tsize, img = miniBlock, speedY = 0}); --top-right
			table.insert(miniBlocks, {x = ((right-1)*tsize+10)-4, y = ((up-1)*tsize-6), img = miniBlock, speedY = 0}); --bottom-right
			table.insert(miniBlocks, {x = ((right-1)*tsize)-4, y = ((up-1)*tsize-6), img = miniBlock, speedY = 0}); --bottom-left
		end
	end
end

--Function to control Ai 
function moveEnemies(dirx, diry, enemie, collisionmap, Map, object)
object = player or object;
collisionmap = layer2;
Map = map;

	getObjectCorners(enemie.x+enemie.speedX*dirx, enemie.y, enemie);
	if enemie.alive and enemie.startFrame ~= 2 then
		if (dirx == 1) then
			if (collisionmap[down][right] == 0 and collisionmap[up][right] == 0) then
				enemie.x = enemie.x + (dirx * enemie.speedX);
				fall(enemie);
			else
				enemie.x = ((right-1) * tsize) - enemie.width;
				enemie.direction = 0;
			end	
		end
		if (dirx == -1) then
			if (collisionmap[down][left] == 0 and collisionmap[up][left] == 0) then	
				enemie.x = enemie.x + (dirx * enemie.speedX);
				fall(enemie);
			else
				enemie.x = (left * tsize);
				enemie.direction = 1;
		end
		if (diry == 1) then
		    getObjectCorners(enemie.x, enemie.y-enemie.speedY*diry, enemie);
			if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then	
				enemie.y = enemie.y - enemie.speedY*diry;
			else
				enemie.y = ((down-1) * tsize) - enemie.height;
				enemie.jump = false;
				enemie.speedY = 0;
			end
		end
	end
		
		--Check collision with player object.
		if (object.x + object.width > enemie.x) and (object.x < enemie.x + enemie.width) and (object.y + object.height > enemie.y) and object.jump and enemie.startFrame ~= 2 then
			object.speedY = 6;
			object.jump = true;
			enemie.startFrame = 2;
		end
	end
end

--Function to move the Ai and update it's postition
function updateAi(object)
object = player or object;
local maxTime = 10;

	for e, enemie in ipairs(enemies) do
		if object.x + 480 > enemie.x and enemie.startFrame ~= 2 and enemie.alive then
			pge.gfx.animateadvanced(enemie, maxTime, enemie, enemie, true);
		--Direction, 0 = left, 1 = right;
			if enemie.direction == 0 then
				moveEnemies(-1, 0, enemie);
			elseif enemie.direction == 1 then
				moveEnemies(1, 0, enemie);
			end
		end
		
		--[[If player collides with enemie then it'll change to a 'dead' sprite. That'll start a counter off which will then 
		    reach a certain int & then it'll stop displaying the enemie.
		]]--
		if enemie.startFrame == 2 then
			enemie.counter = enemie.counter +1;
			
			if enemie.counter > 60 then
				enemie.alive = false;
			end
		end

		if enemie.jump then
			jump(enemie);
		end
		
		--Clean up 
		if enemie.y > 340 then
			table.remove(enemie, e);
		elseif object.x + 480 < enemie.x then
			table.remove(enemie, e);
		elseif not enemie.alive then
			table.remove(enemie, e);
		end
	end
end

--Function to make our objects 'Jump'
function jump(object, grav)
grav = 0.5 or grav;
	object.speedY = object.speedY - grav;

	if (object.speedY < 0) then
		movePlayer(object, 0, 1);
	elseif (object.speedY > 0) then
		movePlayer(object, 0, -1);
	end
end

--Function to animate the player, made by myself. from a mini-lib i wrote
function pge.gfx.animateadvanced(ATime, MaxTimePerFrame, SFrame, EFrame, Constant)
Constant = Constant;  ---wheter to keep doing the animation or not.
local bool = true; 
	if bool then  
		ATime.AnimTime = ATime.AnimTime+1;  ---increase time.

		if ATime.AnimTime >= MaxTimePerFrame then   ---if more than MaxTimePerFrame then reset time, add 1 to the startiong frame. hence animation
			ATime.AnimTime = 0;
			SFrame.startFrame = SFrame.startFrame+1;
			
			if Constant and SFrame.startFrame >= EFrame.endFrame  then   ---if constant & we've reached the last frame then reset it.
				SFrame.startFrame = SFrame.oldFrame;   
			elseif not Constant then  ----if not constant then when anim finishes, stop animating.
				ATime.AnimTime = 0;
				SFrame.startFrame = -1;
			end
		end
	end
end

--Function to make our player 'Fall'
function fall(object, collisionmap)
collisionmap = layer2;
	if not object.jump then
		getObjectCorners(object.x, object.y +1, object);
			if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then
				object.speedY = -0.5;
				object.jump = true;
		    end
	  end
end

--Function to call main PGE functions at start-up.
function startMainFunctions()
	pge.controls.update();
	pge.gfx.startdrawing();
	pge.gfx.clearscreen();
end

function animateMainChar(object)
object = player or object;
local animate = false;

--if not doing anything reset the offset
	if not object.jump and not pge.controls.held(PGE_CTRL_LEFT) and not pge.controls.held(PGE_CTRL_RIGHT) and pge.controls.analogx() < 99 and pge.controls.analogx() > -99 and not object.dead then
		offset = 0;
		currentSpirteEnd = 15;
		
		if direction == Right then
			currentSpirte = StandingR;
		elseif direction == Left then
			currentSpirte = StandingL;
		end
	end

--if dead change spirte 
	if object.dead then
		offset = 16;
		currentSpirteEnd = 17;
		currentSpirte = Dead;
	end
		
--if jumping then change sprite.
	if object.jump and not object.dead then
		if direction == Right then
			currentSpirte = JumpingR;
			currentSpirteEnd = 19;
			offset = 8;
		elseif direction == Left then
			currentSpirte = JumpingL;
			currentSpirteEnd = 19;
			offset = 12;
		end
	end

--if pressing right then animate
	if pge.controls.held(PGE_CTRL_RIGHT) or pge.controls.analogx() > 99 then
		if not object.jump and not object.dead then
			pge.gfx.animateadvanced(MarioAnimRight, MaxTime, MarioAnimRight, MarioAnimRight, true);
			currentSpirte = MarioAnimRight.startFrame;
			animate = true;
		elseif not animate then
			MarioAnimRight.startFrame = MarioAnimRight.oldFrame; 
		else
			MarioAnimRight.startFrame = MarioAnimRight.oldFrame; --resets the animation if not pressing anything.
		end
	end
--if pressing left then animate
	if pge.controls.held(PGE_CTRL_LEFT) or pge.controls.analogx() < -99 then
		if not object.jump and not object.dead then
			pge.gfx.animateadvanced(MarioAnimLeft, MaxTime, MarioAnimLeft, MarioAnimLeft, true);
			currentSpirte = MarioAnimLeft.startFrame;
			animate = true;
		elseif not animate then
			MarioAnimLeft.startFrame = MarioAnimLeft.oldFrame; 
		else
			MarioAnimLeft.startFrame = MarioAnimLeft.oldFrame; --resets the animation if not pressing anything.
		end
	end 

--drifferent size spirtes, so i manual set the offset & end so that it doesn't clip half of the image. Not my fault but it is ugly.
 if animate then
		if currentSpirte == 2 then
			offset = 0;
			currentSpirteEnd = 15;		
		elseif currentSpirte == 3 then
			offset = 0;
			currentSpirteEnd = 16;
		elseif currentSpirte == 4 then
			offset = 1;
			currentSpirteEnd = 18;
		elseif currentSpirte == 5 then
			offset = 4;
			currentSpirteEnd = 15;
		elseif currentSpirte == 6 then
			offset = 3;
			currentSpirteEnd = 17;
		elseif currentSpirte == 7 then
			offset = 5;
			currentSpirteEnd = 18;
		end
	end
end	

--Function to draw a Map
function DrawMap()
--Set the blitting target
texture:settarget();
--Draws background
pge.gfx.clearscreen(backCol);
--Activates our tilesheet
tilesheet:activate();

	if currentFrame == frameSkip then
		TileOnY = math.floor(scr_top/tsize);
		TileOnScreenH = math.floor(TileOnY + 272/tsize+1);
		TileOnX = math.floor(scr_left/tsize);
		TileOnScreenW = math.floor(TileOnX + 480/tsize+1);
	
		if TileOnScreenW > mapX then
			TileOnScreenW = mapX;
		end
		if TileOnScreenH > mapY then
			TileOnScreenH = mapY;
		end

		for y = TileOnY+1,TileOnScreenH do
			for x = TileOnX+1,TileOnScreenW do
				tile = map[y][x];
					--draw tiles
					if tile > 0 and tile < maxTilesPerRow then
						tilesheet:draw((x*tsize-tsize)-scr_left,(y*tsize-tsize)-scr_top, tsize, tsize, tile * tsize, 0, tsize, tsize, 0, 255);
					end
					if tile > maxTilesPerRow then
						tilesheet:draw((x*tsize-tsize)-scr_left,(y*tsize-tsize)-scr_top, tsize, tsize, (tile-(maxTilesPerRow+1)) * tsize, 16, tsize, tsize, 0, 255);
					end
			end
		end
		
	-- Revert to drawing directly to the screen
	pge.gfx.rendertoscreen()
	texture:activate();
	texture:draweasy(0,0);
	
	if #miniBlocks ~= nil then
		for i, block in ipairs(miniBlocks) do
			drawObject(block.x-scr_left, block.y-scr_top, block.img);
		end
	end
	
	if currentFrame >= frameSkip then
		currentFrame = 1;
	end
end
	    
	currentFrame = currentFrame + 1; ---increments the current frame
end

--Function to draw objects to the screen
function drawObject(X, Y, Img, R, A)
R = 0 or R;
A = 255 or A;
	pge.texture.activate(Img)
	pge.texture.draweasy(Img, X, Y, R, A);
end

--Function to draw objects to the scree, advanced.
function drawObjectadvanced(Img, X, Y, Width, Height, StartX, StartY, EndX, EndY, R, A)
R = 0 or R;
A = 255 or A;
	pge.texture.activate(Img);
	pge.texture.draw(Img, X, Y, Width, Height, StartX, StartY, EndX, EndY, R, A);
end

--Function to debug Print ("for Debugging")
function debugPrint(X, Y, colour, str)	
	verdana10:print(X, Y, colour, str);
end

--Function to create a screenshot of the screen ("for Debugging")
function debugShot()
	if pge.controls.pressed(PGE_CTRL_START) then
		screeNum = screeNum + 1;
		pge.gfx.screenshot("Screenie"..screeNum..".jpeg");
	end
end

--Function to call ending PGE functions at the end of each loop.
function endMainFunctions()
	pge.gfx.enddrawing();
	pge.gfx.swapbuffers();
end

