-------------[[[[[Mario Engine]]]]]-------------
-------------[[[[[Written by dan369]]]]]-------------
-------------[[[[[For PSP]]]]]-------------

--[[
This is a platformer engine that i'm writing to make a platform game, i want it to be as re-usable as possible. I call it the Mario Engine because
Mario is the good of all platforming games, it set the standard. So thats the reason behind the name choice. It will be written for the PGE API.
It will use Tilemaps, and tile-based collision for the basic features. Another aspect is to make it as easy as possible to create a level.

Update 24th august 09:
Succesfully started to create a 2d mario clone, using the above features & trying to stick to the criteria above.
Most things done, first release is incoming. Adding polishing touches now & the last few features before a release.
Release expected beore i finsh the 6 weeks holidays.
]]--

----Load Tilesheet
tilesheet = pge.texture.load("Content/GFX/tileset.png", PGE_RAM);
miniBlock = pge.texture.load("Content/GFX/mini-block.png", PGE_RAM);
miniCoins = pge.texture.load("Content/GFX/mini-coins.png", PGE_RAM);

----Load Sounds
Jump = pge.wav.load("Content/SFX/jump.wav");
die = pge.wav.load("Content/SFX/die.wav");
smash = pge.wav.load("Content/SFX/smash.wav");
stomp = pge.wav.load("Content/SFX/stomp.wav");
pause = pge.wav.load("Content/SFX/pause.wav");
warning = pge.wav.load("Content/SFX/warning.wav");
musicIsPlaying = false;
warningPlayed = false;
death = false;
counter = 0;

----Intial variables used for Tilemap
scr_left = 0; ---screen left offset
scr_top = 0;  ---screen top offset
scrSpeed = 2; --scroll speed
currentFrame = 1; ---current frame
frameSkip = 2; ---frameskip
tsize = 16; ---tile size
screeNum = 0;  ---screenshot number
maxTilesPerRow = (tilesheet:width() / tsize) - 1; 

--Enemie constants
enemyTypeGoomba = 3;
enemyTypeKoopa = 4;

--Create table to hold mini-block info.
miniBlocks = {};

--Create color
backCol = pge.gfx.createcolor(92, 148, 252); 

--Gamestate Constants
Start = 0;
Play = 1;
Pause = 2;
State = Start;

--Tile constants
QuestionBlock = 7;

----Animation variables, for our animations
--Constants again, interchange depending on current postition/status.
Right = 1;   
Left = 0;
StandingL = 0; 
StandingR = 1;
JumpingR = 8;
JumpingL = 9;
WalkingR = 2;
WalkingL = 5;
Dead = 10;
TurningRight = 11;
TurningLeft = 12;
GrapRight = 13;
GrapLeft = 14;
direction = Right;  ---holds the current direction
turning = false; --bool for turning, to slow the wlking/running down when changing direction
oldDirection = direction;  --holds the last direction the player is moving in each frame;
currentSpirte = StandingR;  ---holds the current spirte (where to start clipping from)
currentSpirteEnd = 15;    ---this is the width of our image(to end the clipping) i do it manually because of drifferent sized spirtes.

--Mario Animation tables, needed for my animation mini-libray.
MarioAnimRight = {AnimTime = 0, startFrame = WalkingR, endFrame = 5, oldFrame = WalkingR};   ---right animation table
MarioAnimLeft = {AnimTime = 0, startFrame = WalkingL, endFrame = 8, oldFrame = WalkingL};    ---left animation table
MaxTime = 6;   ---MaxTime between frames.
offset = 0;  ---offset required for certain images.

--Table to hold animtion blocks info.
animBlocks = {};

--Level Info for HUD
OverWorld = 1;
UnderWorld = 2;
current = {level = "1-1", lType = OverWorld, Time = pge.timer.create()};
totalTime = 400;
splashCounter = 0;

--Mini-coins animation table.
MiniCoins = {AnimTime = 0, startFrame = 0, endFrame = 3, oldFrame = 0};

--Checkpoint table
Checkpoint = {};

--Flag table
flag = {};

--Function to create the 'player' table.
function createPlayerObject(X, Y, Width, Height, SpeedX, SpeedY, Jump, Img)
	if playerChose == 0 then
		Img = pge.texture.load("Content/GFX/mario.png", PGE_RAM);
	elseif playerChose == 1 then
		Img = pge.texture.load("Content/GFX/luigi.png", PGE_RAM);
	end
	
	player = {};
	player = {x =  X, y = Y, img = Img, width = Width, height = Height, speedX = SpeedX, speedY = SpeedY, jump = Jump, dead = false, lives = 3, maxSpeed = 2, coins = 00, score = 0, win = false};
end

--Function to load a Map
function loadMap(map)
	dofile(map);
end

--Function to load a Collision File
function loadCollisionMap(map)
	dofile(map);
end

--Function to create the 'enemie' objects array. Note: Must be called AFTER loading the collision map! >.>
function createEnemieObjectsAndCheckPoints(colMap)
colMap = layer2 or colMap;

	--load enemie spritesheets
	goomba = pge.texture.load("Content/GFX/goomba.png", PGE_RAM);
	--create table constructor
	enemies = {};
	--loop through file, inserting enemie objects into the array.
	for i, row in ipairs(colMap) do
		for j, tile in ipairs(row) do
			if tile == 3 then
				table.insert(enemies, {x = tsize * (j-1), y = (tsize * (i-1))-2, width = tsize, height = 18, speedX = 1, speedY = 0, direction = 0, jump = false, currentImg = goomba, AnimTime = 0, startFrame = 0, endFrame = 2, oldFrame = 0, Type = enemyTypeGoomba, alive = true, counter =0});
				--perform clean-up
				colMap[i][j] = 0;
			end
			if tile == 5 then
				table.insert(Checkpoint,{x = tsize * (j-1), done = false});
				--perform clean-up
				colMap[i][j] = 0;
			end
		end
	end
end

--Function to create the 'enemie' objects array. Note: Must be called AFTER loading the collision map! >.>
function createAnimTileObjects(Map)
Map = map or Map;

	--loop through file, inserting enemie objects into the array.
	for i, row in ipairs(Map) do
		for j, tile in ipairs(row) do
			if tile == 1 then
				table.insert(animBlocks, {x = j, y = i, AnimTime = 0, startFrame = 1, endFrame = 3, oldFrame = 1});
			end
			if tile == 36 then
				table.insert(flag, {x = (tsize * (j-1)) + 7, y = tsize * (i-1), done = false });
				--clean-up flag, it'll get blitted later.
				Map[i][j] = 0;
			end
		end
	end
end

--Function to detect Movement/Jumping
function detectButtons(object)
oldDirection = direction;
if not object.dead and not object.win then
	if pge.controls.held(PGE_CTRL_RIGHT) or pge.controls.analogx() > 99 then
		direction = Right;
		movePlayer(object, 1, 0);
		scrSpeed = 2;
	elseif pge.controls.held(PGE_CTRL_LEFT) or pge.controls.analogx() < -99 then
		direction = Left;
		movePlayer(object, -1, 0);	
	end

	--if holding the right trigger then change max speed, to fake running.
	if pge.controls.held(PGE_CTRL_RTRIGGER) then
		object.maxSpeed = 3;
		scrSpeed = 3;
		MaxTime = 3;
	else 
		object.maxSpeed = 2;
		scrSpeed = 2;
		MaxTime = 6;
	end
	
	if object.speedX > object.maxSpeed then
		object.speedX = object.maxSpeed;
	end
	
	--if changed direction then reset the speed.
	if direction ~= oldDirection and not object.jump and object.speedX == 2 then
		turning = true;
	elseif direction ~= oldDirection and not object.jump and object.speedX == 3 then
		turning = true;
	end
		
	--if changed direction then reset the speed.
	if turning then
		object.speedX = object.speedX - 0.6;
		
		if object.speedX <= 0 then
			turning = false;
			object.speedX = 0;
		end
	end

	if pge.controls.pressed(PGE_CTRL_CROSS) and not object.jump then
		object.jump = true;
		Jump:play();
		object.speedY = 8.5;
	end
end

	if object.jump and not object.win then
		jump(object);
	end
	if object.y <= 3 then
		object.y = 3;
		object.speedY = -0.5;
	end
	if object.y > 280 and not death then
		object.dead = true;
		object.jump = true;
		object.speedY = 10;
		death = true;
	end
	if object.x <= 0 then
		object.x = 0;
	end
	if scr_left > (#map[1] * tsize - 480) then
		scr_left = (#map[1] * tsize - 480);
	end
end

--Function to get the object corners used in Collision
function getObjectCorners(x, y, object)
	down = math.floor((y + object.height - 1) / tsize)+1;
	up = math.floor(y / tsize)+1;
	left = math.floor(x / tsize)+1;
	right = math.floor((x + object.width - 1) / tsize)+1;
end

--Function to update The Player's X/Y postition
function movePlayer(object, dirx, diry, collisionmap, Map)
collisionmap = layer2;
Map = map;
local collide = false;
local collide1 = false;

--update postition
if not object.dead and object.y < 245 then
	 object.x = object.x + object.speedX * dirx;
	 if object.speedY > 0 then
		 object.y = object.y + object.speedY * diry; 
	 elseif object.speedY < 0 then
		 object.y = object.y - object.speedY * diry; 
	 end		    

	getObjectCorners(object.x, object.y, object);
	if (dirx == 1) then
	  object.speedX = object.speedX + 0.3;
		if (collisionmap[down][right] == 0 and collisionmap[up][right] == 0) then
			fall(object);
			if (object.x-scr_left) > 280 then
				scr_left = scr_left + scrSpeed;
			end
		else
			object.x = ((right-1) * tsize) - object.width;
		end	
	end
	if (dirx == -1) then
	  object.speedX = object.speedX + 0.3;
		if (collisionmap[down][left] == 0 and collisionmap[up][left] == 0) then	
			fall(object);
		else
			object.x = (left * tsize);
		end
	end
	
	if (diry == 1) then
		if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then	
		else
			object.y = ((down-1) * tsize) - object.height;
			object.jump = false;
			object.speedY = 0;
		end
	end
	if (diry == -1) then
		if (collisionmap[up][left] == 0 and collisionmap[up][right] == 0) then	   
		else
			object.y = (up * tsize);
			object.speedY = -1;
		end
	end
else
	object.y = object.y - object.speedY;
end

	   if (diry == -1) and not object.dead then
			if (collisionmap[up][right] == 2) then	
				Map[up][right] = 0;		
				collisionmap[up][right] = 0;
				smash:play();
				collide1 = true;
			elseif (collisionmap[up][left] == 2) then
				Map[up][left] = 0;
				collisionmap[up][left] = 0;
				smash:play();
				collide = true;
			else
				collide = false;
				collide1 = false;
			end
			
		if #animBlocks > 0 then
		  if (diry == -1) and not object.dead then
			if (collisionmap[up][right] == 4) then	
				Map[up][right] = QuestionBlock;
			elseif (collisionmap[up][left] == 4) then
				Map[up][left] = QuestionBlock;
			   end
			end
		end
	end
	
	--Check if player is tounching the flag
	if object.x >= flag[1].x + 7 then
		object.win = true;
	end
	
		--Insert miniBlocks in correct places depending on whether the player collides on the right/left underneath side of block.
		if collide then
			getObjectCorners(object.x, object.y, object);
			table.insert(miniBlocks, {x = ((left-1)*tsize), y = (up-2)*tsize, img = miniBlock, speedY = 7, speedX = -0.4});  --top-left
			table.insert(miniBlocks, {x = ((left-1)*tsize+10), y = (up-2)*tsize, img = miniBlock, speedY = 7, speedX = 0.4}); --top-right
			table.insert(miniBlocks, {x = ((left-1)*tsize+10), y = ((up-1)*tsize-6), img = miniBlock, speedY = 5, speedX = 0.4}); --bottom-right
			table.insert(miniBlocks, {x = ((left-1)*tsize), y = ((up-1)*tsize-6), img = miniBlock, speedY = 5, speedX = -0.4}); --bottom-left
		elseif collide1 then
			getObjectCorners(object.x, object.y, object);
			table.insert(miniBlocks, {x = ((right-1)*tsize)-4, y = (up-2)*tsize, img = miniBlock, speedY = 7, speedX = -0.4}); --top-left
			table.insert(miniBlocks, {x = ((right-1)*tsize+10)-4, y = (up-2)*tsize, img = miniBlock, speedY = 7, speedX = 0.4}); --top-right
			table.insert(miniBlocks, {x = ((right-1)*tsize+10)-4, y = ((up-1)*tsize-6), img = miniBlock, speedY = 5, speedX = 0.4}); --bottom-right
			table.insert(miniBlocks, {x = ((right-1)*tsize)-4, y = ((up-1)*tsize-6), img = miniBlock, speedY = 5, speedX = -0.4}); --bottom-left
		end
end

--Function to control Ai 
function moveEnemies(dirx, diry, enemie, collisionmap, Map)
object = player or object;
collisionmap = layer2;
Map = map;

	if enemie.alive and enemie.startFrame ~= 2 and enemie.y < 245 then
	  getObjectCorners(enemie.x+enemie.speedX*dirx, enemie.y, enemie);
		if (dirx == 1) then
			if (collisionmap[down][right] == 0 and collisionmap[up][right] == 0) then
				enemie.x = enemie.x + (dirx * enemie.speedX);
				fall(enemie);
			else
				enemie.x = ((right-1) * tsize) - enemie.width;
				enemie.direction = 0;
			end	
		end
		if (dirx == -1) then
			if (collisionmap[down][left] == 0 and collisionmap[up][left] == 0) then	
				enemie.x = enemie.x + (dirx * enemie.speedX);
				fall(enemie);
			else
				enemie.x = (left * tsize);
				enemie.direction = 1;
			end
		end
		if (diry == 1) and enemie.y < 225 then
		    getObjectCorners(enemie.x, enemie.y-enemie.speedY*diry, enemie);
			if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then	
				enemie.y = enemie.y - enemie.speedY*diry;
			else
				enemie.y = ((down-1) * tsize) - enemie.height;
				enemie.jump = false;
				enemie.speedY = 0;
				end
			end
	else
		enemie.y = enemie.y - enemie.speedY;
	end
end

--Function to move the Ai and update it's postition
function updateAi(object)
object = player or object;
local maxTime = 10;

if #enemies > 0 then
	for e, enemie in ipairs(enemies) do
		if object.x + 480 > enemie.x and enemie.startFrame ~= 2 and enemie.alive and enemie.y < 225 then
			pge.gfx.animateadvanced(enemie, maxTime, enemie, enemie, true);
		--Direction, 0 = left, 1 = right;
			if enemie.direction == 0 then
				moveEnemies(-1, 0, enemie);
			elseif enemie.direction == 1 then
				moveEnemies(1, 0, enemie);
			end
		end
		
		--[[If player collides with enemie then it'll change to a 'dead' sprite. That'll start a counter off which will then 
		    reach a certain int & then it'll stop displaying the enemie.
		]]--
		if enemie.startFrame == 2 then
			enemie.counter = enemie.counter +1;
			
			if enemie.counter > 60 then
				enemie.alive = false;
			end
		end

		if enemie.jump then
			jump(enemie);
		end
		
		--Clean up 
		if enemie.y > 300 then
			table.remove(enemies, e);
		elseif object.x > enemie.x + 480 then
			table.remove(enemies, e);
		elseif not enemie.alive then
			table.remove(enemies, e);
		end
		
		--Check enemie collision with player object.
		if (object.y + object.height > enemie.y+2) and (object.y <= enemie.y) and (object.x + object.width > enemie.x) and (object.x < enemie.x + enemie.width) and object.speedY < 0 and object.jump and not object.dead and enemie.startFrame ~= 2 then
			object.speedY = 5;
			object.jump = true;
			stomp:play()
			enemie.startFrame = 2;
		end
		
		if (object.y-2) == enemie.y and object.x + object.width > enemie.x and object.x < enemie.x + enemie.width and not object.dead and enemie.startFrame ~= 2 then
			object.speedY = 10;
			object.jump = true;
			object.dead = true;
			death = true;
		end
	end
		
		if object.dead and not musicIsPlaying then
			die:play();
			object.lives = object.lives - 1;
			musicIsPlaying = true;
		elseif object.dead and musicIsPlaying then
			counter = counter + 1;
		end
		
		if counter >= 150 and State ~= Start then
			State = Start;
			counter = 0;
		end
	end
end

--Function to make our objects 'Jump'
function jump(object, grav)
if not object.dead then
	grav = 0.5 or grav;
else
	grav = 0.35;
end
object.speedY = object.speedY - grav;

	if (object.speedY < 0) then
		movePlayer(object, 0, 1);
	elseif (object.speedY > 0) then
		movePlayer(object, 0, -1);
	end
end

--Function for Checkpoint system
function CheckIfObjectHasPassedCheckpoint(object)
object = player or object 
if #Checkpoint > 0 then
	for c = 1,table.getn(Checkpoint) do
		if object.x + object.width >= Checkpoint[c].x and Checkpoint[c].x <= object.x + object.width then
			Checkpoint[c].done = true
		end
		
			if Checkpoint[c].done then
				cp = Checkpoint[c];
			end
		end
	end
end

--Function to animate the player, made by myself. from a mini-lib i wrote
function pge.gfx.animateadvanced(ATime, MaxTimePerFrame, SFrame, EFrame, Constant)
Constant = Constant;  ---wheter to keep doing the animation or not.
local bool = true; 
	if bool then  
		ATime.AnimTime = ATime.AnimTime+1;  ---increase time.

		if ATime.AnimTime >= MaxTimePerFrame then   ---if more than MaxTimePerFrame then reset time, add 1 to the startiong frame. hence animation
			ATime.AnimTime = 0;
			SFrame.startFrame = SFrame.startFrame+1;
			
			if Constant and SFrame.startFrame >= EFrame.endFrame  then   ---if constant & we've reached the last frame then reset it.
				SFrame.startFrame = SFrame.oldFrame;   
			elseif not Constant then  ----if not constant then when anim finishes, stop animating.
				ATime.AnimTime = 0;
				SFrame.startFrame = -1;
			end
		end
	end
end

--Function to make our player 'Fall'
function fall(object, collisionmap)
collisionmap = layer2;
	if not object.jump then
		getObjectCorners(object.x, object.y +1, object);
			if (collisionmap[down][left] == 0 and collisionmap[down][right] == 0) then
				object.speedY = -0.5;
				object.jump = true;
		    end
	  end
end

--Function to call main PGE functions at start-up.
function startMainFunctions()
	pge.controls.update();
	pge.gfx.startdrawing();
	pge.gfx.clearscreen();
end

function animateMainChar(object)
object = player or object;
local animate = false;

--if not doing anything reset the offset
if not object.dead then
	if not animate then		
		currentSpirteEnd = 15;
		offset = 0;
	
	if direction == Right then
		currentSpirte = StandingR;
		currentSpirteEnd = 15;
		offset = 0;
	elseif direction == Left then
		currentSpirte = StandingL;
		currentSpirteEnd = 15;
		offset = 0;
	end
	
	--if jumping then change sprite.
	if object.jump then
		if direction == Right then
			currentSpirte = JumpingR;
			currentSpirteEnd = 19;
			offset = 8;
		elseif direction == Left then
			currentSpirte = JumpingL;
			currentSpirteEnd = 19;
			offset = 12;
		end
	end
end
	
	--if turning then change to turning sprite 
	if turning then
		if direction == Left then
			currentSpirte = TurningLeft;
			offset = 19;
			currentSpirteEnd = 16;
		elseif direction == Right then
			currentSpirte = TurningRight;
			offset = 18;
			currentSpirteEnd = 16;
		end
	end
	
	--if win, then change to flag animation;
	if object.win then
		if direction == Right then
			currentSpirte = GrapRight;
			offset = 20;
			currentSpirteEnd = 15;
		elseif direction == Left then
			currentSpirte = GrapLeft;
			offset = 20;
			currentSpirteEnd = 15;
		end
	end

--if pressing right then animate
	if pge.controls.held(PGE_CTRL_RIGHT) or pge.controls.analogx() > 99 then
		if not object.jump and not turning and not object.win then
			pge.gfx.animateadvanced(MarioAnimRight, MaxTime, MarioAnimRight, MarioAnimRight, true);
			currentSpirte = MarioAnimRight.startFrame;
			animate = true;
		end
	end
	--if pressing left then animate
	if pge.controls.held(PGE_CTRL_LEFT) or pge.controls.analogx() < -99 then
		if not object.jump and not turning and not object.win then
			pge.gfx.animateadvanced(MarioAnimLeft, MaxTime, MarioAnimLeft, MarioAnimLeft, true);
			currentSpirte = MarioAnimLeft.startFrame;
			animate = true;
		end
	end 
	
	--drifferent size spirtes, so i manual set the offset & end so that it doesn't clip half of the image. Not my fault but it is ugly.
	if currentSpirte == 2 then
		offset = 0;
		currentSpirteEnd = 15;		
	elseif currentSpirte == 3 then
		offset = 0;
		currentSpirteEnd = 16;
	elseif currentSpirte == 4 then
		offset = 1;
		currentSpirteEnd = 18;
	elseif currentSpirte == 5 then
		offset = 4;
		currentSpirteEnd = 15;
	elseif currentSpirte == 6 then
		offset = 3;
		currentSpirteEnd = 17;
	elseif currentSpirte == 7 then
		offset = 5;
		currentSpirteEnd = 18;
end

else
		currentSpirte = Dead;
		offset = 16;
		currentSpirteEnd = 17;
	end
end	

--Function to draw a Map
function DrawMap()
--Set the blitting target
texture:settarget();
--Draws background
pge.gfx.clearscreen(backCol);
--Activates our tilesheet
tilesheet:activate();

	if currentFrame == frameSkip then
		TileOnY = math.floor(scr_top/tsize);
		TileOnScreenH = math.floor(TileOnY + 272/tsize+1);
		TileOnX = math.floor(scr_left/tsize);
		TileOnScreenW = math.floor(TileOnX + 480/tsize+1);
	
		if TileOnScreenW > mapX then
			TileOnScreenW = mapX;
		end
		if TileOnScreenH > mapY then
			TileOnScreenH = mapY;
		end

		for y = TileOnY+1,TileOnScreenH do
			for x = TileOnX+1,TileOnScreenW do
				tile = map[y][x];				
					--draw tiles
					if tile > 0 and tile < maxTilesPerRow then
						tilesheet:draw((x*tsize-tsize)-scr_left,(y*tsize-tsize)-scr_top, tsize, tsize, tile * tsize, 0, tsize, tsize, 0, 255);
					end
					if tile > maxTilesPerRow then
						tilesheet:draw((x*tsize-tsize)-scr_left,(y*tsize-tsize)-scr_top, tsize, tsize, (tile-(maxTilesPerRow+1)) * tsize, 16, tsize, tsize, 0, 255);
					end
			end
		end
		
	if #miniBlocks > 0 then
		for b, block in ipairs(miniBlocks) do
			drawObject(block.x-scr_left, block.y-scr_top, block.img);
		end
	end
	
	--draw mini coin
	drawObjectadvanced(miniCoins, 125, 27, 7, 10, MiniCoins.startFrame * 7, 0, 7, 10);
	
	--draw flag
	if player.x + 480 > flag[1].x then
		drawObjectadvanced(tilesheet, (flag[1].x)-scr_left,(flag[1].y)-scr_top, tsize, tsize, (36-(maxTilesPerRow+1)) * tsize, 16, tsize, tsize, 0, 255);
	end
		
	-- Revert to drawing directly to the screen
	pge.gfx.rendertoscreen()
	texture:activate();
	texture:draweasy(0,0);
	
	if currentFrame >= frameSkip then
		currentFrame = 1;
	end
end
	    
	currentFrame = currentFrame + 1; ---increments the current frame
end

--Function to draw objects to the screen
function drawObject(X, Y, Img, R, A)
R = 0 or R;
A = 255 or A;
	pge.texture.activate(Img)
	pge.texture.draweasy(Img, X, Y, R, A);
end

--Function to draw objects to the scree, advanced.
function drawObjectadvanced(Img, X, Y, Width, Height, StartX, StartY, EndX, EndY, R, A)
R = 0 or R;
A = 255 or A;
	pge.texture.activate(Img);
	pge.texture.draw(Img, X, Y, Width, Height, StartX, StartY, EndX, EndY, R, A);
end

--Function to print
function debugPrint(X, Y, colour, str)	
	verdana10:print(X, Y, colour, str);
end

--Function to draw enemies & player
function drawPlayersAndEnemies()
local tempX = math.floor(player.x);
	for i, enemie in ipairs(enemies) do
		if enemie.alive then
			drawObjectadvanced(goomba, enemie.x-scr_left, enemie.y-scr_top, enemie.width+2, enemie.height, (enemie.startFrame * (enemie.width+2)), 0, (enemie.width+2), (enemie.height));
		end
	end

	if player.jump and direction == Right then
		drawObjectadvanced(player.img, tempX-scr_left-4, player.y-scr_top-2, currentSpirteEnd, (player.height+2), (currentSpirte * 15)+offset, 0, currentSpirteEnd, (player.height+2));
	elseif player.jump and direction == Left then
		drawObjectadvanced(player.img, tempX-scr_left-2, player.y-scr_top-2, currentSpirteEnd, (player.height+2), (currentSpirte * 15)+offset, 0, currentSpirteEnd, (player.height+2));
	else
		drawObjectadvanced(player.img, tempX-scr_left, player.y-scr_top-2, currentSpirteEnd, (player.height+2), (currentSpirte * 15)+offset, 0, currentSpirteEnd, (player.height+2));
	end
end

--Function to update/draw/animate blocks
function updateBlocks(grav, object)
grav = 0.5 or grav;
object = player or object;
local maxTime = 14;

	if #miniBlocks > 0 then
		for b, block in ipairs(miniBlocks) do
			
			--update block postitions
			block.speedY = block.speedY - grav;
			block.y = block.y - block.speedY;
			block.x = block.x + block.speedX;
			
			if block.y > 300 then
				table.remove(miniBlocks, b);
			end
		end
	end
	
	if #animBlocks > 0 then
		for i, anim in ipairs(animBlocks) do
			if object.x + 480 > anim.x and map[anim.y][anim.x] ~= QuestionBlock then
				pge.gfx.animateadvanced(anim, maxTime, anim, anim, true);
				map[anim.y][anim.x] = anim.startFrame;
			else
				table.remove(animBlocks, i);
			end
		end
	end
end

function updateTime(maxTime, object)
object = player or object;
maxTime = 400 or maxTime;
local death = false;
totalTime = math.floor(maxTime - (current.Time:peekdelta()));

	if totalTime == 0 then
		death = true;
	end
	
	if totalTime == 70 and not warningPlayed then
		warning:play();
		warningPlayed = true;
	end
	
	if death and not object.dead then
		object.speedY = 10;
		object.jump = true;
		object.dead = true;
	else
		death = false;
	end
end

--HuD function
function updateHUD()
local maxTime = 10;
	--animate mini coin
	pge.gfx.animateadvanced(MiniCoins, maxTime, MiniCoins, MiniCoins, true);
	--activate the font
	verdana10:activate();
	if playerChose == 0 then
		debugPrint(45, 20, white, "MARIO");
	elseif playerChose == 1 then
		debugPrint(45, 20, white, "LUIGI");
	end
	debugPrint(45, 30, white, ""..player.score);
	debugPrint(135, 30, white, "x "..player.coins);
	debugPrint(230, 20, white, "WORLD");
	debugPrint(230, 30, white, current.level);
	debugPrint(350, 20, white, "TIME");
	debugPrint(350, 30, white, ""..totalTime);
end

--GameState function
function checkState(object)
object = player or object;
	if State == Start then
		splashCounter = splashCounter + 1;
		totalTime = 400;
		object.dead = false;
		musicIsPlaying = false;
		warningPlayed = false;
		death = false;
		current.Time = pge.timer.create();
			
	if #Checkpoint > 0 then
		for c = 1, #Checkpoint do
			if Checkpoint[c].done then
				object.x = cp.x + object.width;
				object.y = 20;
				object.speedY = 0;
				
				if scr_left > (#map[1] * tsize - 480) then
					scr_left = (#map[1] * tsize - 480);
				else
					scr_left = cp.x;
				end
				
			end
		end
	else
		object.x = 10;
		object.y = 20;
		object.speedY = -1;
		scr_left = 0;	
	end
		
		--activate the font
		verdana10:activate();
		debugPrint(210, 130, white, " X "..object.lives);
		drawObjectadvanced(object.img, 195, 122, 15, object.height, (StandingR * 15), 0, 15, object.height);
	
		if splashCounter >= 150 then
			State = Play;
			splashCounter = 0;
		end
	end
	
	if State == Play then
		detectButtons(player);
		animateMainChar();	
		updateAi(); 
		updateTime();
		updateBlocks();
		CheckIfObjectHasPassedCheckpoint();
		
		if object.win then
			object.x = flag[1].x + 5;
		
		if object.y >= 208 then
		else
			object.y = object.y + 1.3;
		end
		
		if flag[1].y > (object.y) then
			direction = Left;
			object.x = flag[1].x + 18;
			dofile("win.lua");
		else
			flag[1].y = flag[1].y + 1;
		end
	end
	
		if pge.controls.pressed(PGE_CTRL_START) then
			State = Pause;
			pause:play();
		end
	end
	
	if State ~= Start then
		DrawMap();
		drawPlayersAndEnemies();
	end
	
	if State == Pause then
		if pge.controls.pressed(PGE_CTRL_START) then
			State = Play;
		end
	end
end

--Function to create a screenshot of the screen ("for Debugging")
function debugShot()
	if pge.controls.pressed(PGE_CTRL_SELECT) then
		screeNum = screeNum + 1;
		pge.gfx.screenshot("Screenie"..screeNum..".jpeg");
	end
end

--Function to call ending PGE functions at the end of each loop.
function endMainFunctions()
	pge.gfx.enddrawing();
	pge.gfx.swapbuffers();
end

